︠db934717-fcba-4795-96b4-33c223398b9dis︠
%hide
%html
<p><strong>Function definition:</strong> <span style="font-family: courier new,courier;">dde_solve(dde, statevar, delayedvars, history, tmax, timestep)</span></p>
<p><strong>Description:</strong> Numerically integrate a delay differential equation</p>
<p style="margin-left: 30px;">A single-variable delay differential equation is an equation of the form <em>X</em>'(<em>t</em>) = <em>f</em>(<em>X</em>, <em>X</em><sub>&tau;<sub>1</sub></sub>, <em>X</em><sub>&tau;<sub>2</sub></sub>, ... , <em>X</em><sub>&tau;<sub>n</sub></sub>) where on the right hand side <em>X</em> denotes <em>X</em>(<em>t</em>) and each <em>X</em><sub>&tau;</sub> denotes <em>X</em>(<em>t</em> - &tau;) for some constant delay &tau;.</p>
<p><strong>Inputs:</strong></p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">dde</span>: The right hand side of the delay differential equation</p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">statevar</span>: The state variable to use (<em>X</em> in the example above)</p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">delayedvars</span>: A dictionary whose keys are the symbolic variables that appear in the DDE expression, which represent the value of the state variable some time ago, and whose values are the actual delays. For example, if the DDE is <em>X</em>'(<em>t</em>) = <em>X</em>(<em>t</em>) - X(<em>t</em> - 5) + <em>X</em>(<em>t</em> - 17.4) the <span style="font-family: courier new,courier;">dde</span> argument might be <span style="font-family: courier new,courier;">X - Xtau + Xtau2</span>, in which case this argument would be <span style="font-family: courier new,courier;">{Xtau: 5, Xtau2: 17.4}</span>. Note that this means before calling this function, all three of <span style="font-family: courier new,courier;">X</span>, <span style="font-family: courier new,courier;">Xtau</span>, and <span style="font-family: courier new,courier;">Xtau2</span> should have been declared as symbolic variables: <span style="font-family: courier new,courier;">var("X, Xtau, Xtau2")</span></p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">history</span>: A Sage expression for the history, as a function of <span style="font-family: courier new,courier;">t</span> (time)</p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">tmax</span>: Integrate from <em>t</em> = 0 to this time</p>
<p style="margin-left: 30px;"><span style="font-family: courier new,courier;">timestep</span>: The size of each time step (the smaller, the more accurate)</p>
<p><strong>Output:</strong> A list of the values of the state variable (<em>X</em> in the example above) at each time step from <em>t</em> = 0 to <span style="font-family: courier new,courier;">tmax</span></p>
<p><strong>Examples:</strong></p>
<p style="margin-left: 30px;">A version of Mackey-Glass, using a constant 0 history:</p>
<p style="margin-left: 60px;"><span style="font-family: courier new,courier;"> var("X, X_t")<br /> sol = dde_solve(1 - X * X_t^5/(1 + X_t^5), X, {X_t: 8}, history=0, tmax=400, timestep=0.1)<br /> list_plot(zip(srange(0, 400, 0.1), sol), plotjoined=True)</span></p>
<p style="margin-left: 30px;">The same, but with an oscillating history function, giving weird transients that give way to a periodic steady state:</p>
<p style="margin-left: 60px;"><span style="font-family: courier new,courier;"> var("t, X, X_tau")<br /> eq = 1 - X * X_tau^5/(1 + X_tau^5)<br /> sol = dde_solve(eq, X, {X_tau: 8}, 5*cos(t), 400, 0.1)<br /> list_plot(zip(srange(0, 400, 0.1), sol), plotjoined=True)</span></p>

︡57fb1982-118d-4fdd-93b2-bedb6ff15384︡
︠ac5084bc-0417-4eba-9efd-5bde129b5e61as︠
%auto
def dde_solve(dde, statevar, delayedvars, history, tmax, timestep):
    # Check validity of delays. 
    if min(delayedvars.values()) < 0:
        raise ValueError("This function will not work with negative delays. "
            "Consider consulting a fortune teller instead.")
    if any(val<timestep for val in delayedvars.values()):
        raise ValueError("Time step should not be larger than delay.")

    # Set up variables and delays. 
    delayedvars = delayedvars.items()
    dde = dde.subs({v: statevar for v, delay in delayedvars if delay == 0})
    delayedvars = [(v, delay) for v, delay in delayedvars if delay != 0]
    allvars = [str(statevar)] + [str(v) for v, delay in delayedvars]
    delays = [delay for v, delay in delayedvars]

    # Set up fast functions. 
    dde_func = fast_float(dde, *allvars)
    history_func = fast_float(history, "t")

    # Adjust the timestep if necessary
    mindelay = min(delays) if delays else timestep
    timestepcorrectionfactor = ceil(timestep / mindelay)
    timestep /= timestepcorrectionfactor

    # A function to perform history lookups. 
    def lookup(t):
        """Does a history lookup at each delay from t, stores result in allvars[1:]"""
        for i, delay in enumerate(delays):
            if t - delay <= 0:
                allvars[i+1] = history_func(t - delay)
            else:
                r = (t - delay) / timestep
                n = floor(r)
                r -= n
                allvars[i+1] = result[n]*(1 - r) + result[n + 1]*r

    # Set up for the first iteration. 
    result = [history_func(0)]
    lookup(0)
    for t in sxrange(0, tmax - timestep, timestep):
        # Compute k1. Note history lookup has already been done. 
        allvars[0] = result[-1]
        k1 = dde_func(*allvars)
        # Compute k2. 
        lookup(t + timestep/2)
        allvars[0] += timestep/2 * k1
        k2 = dde_func(*allvars)
        # Compute k3. Note history lookup has already been done. 
        allvars[0] = result[-1] + timestep/2 * k2
        k3 = dde_func(*allvars)
        # Compute k4. 
        lookup(t + timestep)
        allvars[0] = result[-1] + timestep * k3
        k4 = dde_func(*allvars)
        # Finally, compute the RK4 weighted average. 
        result.append(result[-1] + (k1 + 2*k2 + 2*k3 + k4)/6 * timestep)
    return result[::timestepcorrectionfactor]
︡386941b7-2a28-4cd2-b31d-42321daa7d6d︡{"done":true}︡
︠5392738a-3232-44c7-8b42-20d9dd2a91f8s︠
# Interactive for Mackey-Glass Model
@interact
def respiration (tau = (0.1,0.2,0.001), n = (0.1, 10, 0.01)):
    var("x", "x_tau")
    L = 6
    Vmax = 80
    a = 0.2
    h = 400
    max_time = 200
    tstep = 0.05
    xprime = L - (Vmax * x_tau^n)/(h + x_tau^n) * a * x
    delay = {x_tau: tau}
    sol = dde_solve(xprime,x,delay, history = 0, tmax = max_time, timestep = tstep)
    time = srange(0,max_time,tstep)
    show(list_plot(zip(time, sol), plotjoined = true, axes_labels = ["Time (minutes)","Concentration of Carbon Dioxide "]))
︡eaea5d64-ba28-41e4-a1a1-fdb9849b86da︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"tau","vals":["0.100000000000000","0.101000000000000","0.102000000000000","0.103000000000000","0.104000000000000","0.105000000000000","0.106000000000000","0.107000000000000","0.108000000000000","0.109000000000000","0.110000000000000","0.111000000000000","0.112000000000000","0.113000000000000","0.114000000000000","0.115000000000000","0.116000000000000","0.117000000000000","0.118000000000000","0.119000000000000","0.120000000000000","0.121000000000000","0.122000000000000","0.123000000000000","0.124000000000000","0.125000000000000","0.126000000000000","0.127000000000000","0.128000000000000","0.129000000000000","0.130000000000000","0.131000000000000","0.132000000000000","0.133000000000000","0.134000000000000","0.135000000000000","0.136000000000000","0.137000000000000","0.138000000000000","0.139000000000000","0.140000000000000","0.141000000000000","0.142000000000000","0.143000000000000","0.144000000000000","0.145000000000000","0.146000000000000","0.147000000000000","0.148000000000000","0.149000000000000","0.150000000000000","0.151000000000000","0.152000000000000","0.153000000000000","0.154000000000000","0.155000000000000","0.156000000000000","0.157000000000000","0.158000000000000","0.159000000000000","0.160000000000000","0.161000000000000","0.162000000000000","0.163000000000000","0.164000000000000","0.165000000000000","0.166000000000000","0.167000000000000","0.168000000000000","0.169000000000000","0.170000000000000","0.171000000000000","0.172000000000000","0.173000000000000","0.174000000000000","0.175000000000000","0.176000000000000","0.177000000000000","0.178000000000000","0.179000000000000","0.180000000000000","0.181000000000000","0.182000000000000","0.183000000000000","0.184000000000000","0.185000000000000","0.186000000000000","0.187000000000000","0.188000000000000","0.189000000000000","0.190000000000000","0.191000000000000","0.192000000000000","0.193000000000000","0.194000000000000","0.195000000000000","0.196000000000000","0.197000000000000","0.198000000000000","0.199000000000000","0.200000000000000"],"var":"tau","width":null},{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"n","vals":["0.100000000000000","0.110000000000000","0.120000000000000","0.130000000000000","0.140000000000000","0.150000000000000","0.160000000000000","0.170000000000000","0.180000000000000","0.190000000000000","0.200000000000000","0.210000000000000","0.220000000000000","0.230000000000000","0.240000000000000","0.250000000000000","0.260000000000000","0.270000000000000","0.280000000000000","0.290000000000000","0.300000000000000","0.310000000000000","0.320000000000000","0.330000000000000","0.340000000000000","0.350000000000000","0.360000000000000","0.370000000000000","0.380000000000000","0.390000000000000","0.400000000000000","0.410000000000000","0.420000000000000","0.430000000000000","0.440000000000000","0.450000000000000","0.460000000000000","0.470000000000000","0.480000000000000","0.490000000000000","0.500000000000000","0.510000000000000","0.520000000000000","0.530000000000000","0.540000000000000","0.550000000000000","0.560000000000000","0.570000000000000","0.580000000000000","0.590000000000000","0.600000000000000","0.610000000000000","0.620000000000000","0.630000000000000","0.640000000000000","0.650000000000000","0.660000000000000","0.670000000000000","0.680000000000000","0.690000000000001","0.700000000000001","0.710000000000001","0.720000000000001","0.730000000000001","0.740000000000001","0.750000000000001","0.760000000000001","0.770000000000001","0.780000000000001","0.790000000000001","0.800000000000001","0.810000000000001","0.820000000000001","0.830000000000001","0.840000000000001","0.850000000000001","0.860000000000001","0.870000000000001","0.880000000000001","0.890000000000001","0.900000000000001","0.910000000000001","0.920000000000001","0.930000000000001","0.940000000000001","0.950000000000001","0.960000000000001","0.970000000000001","0.980000000000001","0.990000000000001","1.00000000000000","1.01000000000000","1.02000000000000","1.03000000000000","1.04000000000000","1.05000000000000","1.06000000000000","1.07000000000000","1.08000000000000","1.09000000000000","1.10000000000000","1.11000000000000","1.12000000000000","1.13000000000000","1.14000000000000","1.15000000000000","1.16000000000000","1.17000000000000","1.18000000000000","1.19000000000000","1.20000000000000","1.21000000000000","1.22000000000000","1.23000000000000","1.24000000000000","1.25000000000000","1.26000000000000","1.27000000000000","1.28000000000000","1.29000000000000","1.30000000000000","1.31000000000000","1.32000000000000","1.33000000000000","1.34000000000000","1.35000000000000","1.36000000000000","1.37000000000000","1.38000000000000","1.39000000000000","1.40000000000000","1.41000000000000","1.42000000000000","1.43000000000000","1.44000000000000","1.45000000000000","1.46000000000000","1.47000000000000","1.48000000000000","1.49000000000000","1.50000000000000","1.51000000000000","1.52000000000000","1.53000000000000","1.54000000000000","1.55000000000000","1.56000000000000","1.57000000000000","1.58000000000000","1.59000000000000","1.60000000000000","1.61000000000000","1.62000000000000","1.63000000000000","1.64000000000000","1.65000000000000","1.66000000000000","1.67000000000000","1.68000000000000","1.69000000000000","1.70000000000000","1.71000000000000","1.72000000000000","1.73000000000000","1.74000000000000","1.75000000000000","1.76000000000000","1.77000000000000","1.78000000000000","1.79000000000000","1.80000000000000","1.81000000000000","1.82000000000000","1.83000000000000","1.84000000000000","1.85000000000000","1.86000000000000","1.87000000000000","1.88000000000000","1.89000000000000","1.90000000000000","1.91000000000000","1.92000000000000","1.93000000000000","1.94000000000000","1.95000000000000","1.96000000000000","1.97000000000000","1.98000000000000","1.99000000000000","2.00000000000000","2.01000000000000","2.02000000000000","2.03000000000000","2.04000000000000","2.05000000000000","2.06000000000000","2.07000000000000","2.08000000000000","2.09000000000000","2.10000000000000","2.11000000000000","2.12000000000000","2.13000000000000","2.14000000000000","2.15000000000000","2.16000000000000","2.17000000000000","2.18000000000000","2.19000000000000","2.20000000000000","2.21000000000000","2.22000000000000","2.23000000000000","2.24000000000000","2.25000000000000","2.26000000000000","2.27000000000000","2.28000000000000","2.29000000000000","2.29999999999999","2.30999999999999","2.31999999999999","2.32999999999999","2.33999999999999","2.34999999999999","2.35999999999999","2.36999999999999","2.37999999999999","2.38999999999999","2.39999999999999","2.40999999999999","2.41999999999999","2.42999999999999","2.43999999999999","2.44999999999999","2.45999999999999","2.46999999999999","2.47999999999999","2.48999999999999","2.49999999999999","2.50999999999999","2.51999999999999","2.52999999999999","2.53999999999999","2.54999999999999","2.55999999999999","2.56999999999999","2.57999999999999","2.58999999999999","2.59999999999999","2.60999999999999","2.61999999999999","2.62999999999999","2.63999999999999","2.64999999999999","2.65999999999999","2.66999999999999","2.67999999999999","2.68999999999999","2.69999999999999","2.70999999999999","2.71999999999999","2.72999999999999","2.73999999999999","2.74999999999999","2.75999999999999","2.76999999999998","2.77999999999998","2.78999999999998","2.79999999999998","2.80999999999998","2.81999999999998","2.82999999999998","2.83999999999998","2.84999999999998","2.85999999999998","2.86999999999998","2.87999999999998","2.88999999999998","2.89999999999998","2.90999999999998","2.91999999999998","2.92999999999998","2.93999999999998","2.94999999999998","2.95999999999998","2.96999999999998","2.97999999999998","2.98999999999998","2.99999999999998","3.00999999999998","3.01999999999998","3.02999999999998","3.03999999999998","3.04999999999998","3.05999999999998","3.06999999999998","3.07999999999998","3.08999999999998","3.09999999999998","3.10999999999998","3.11999999999998","3.12999999999998","3.13999999999998","3.14999999999998","3.15999999999998","3.16999999999998","3.17999999999998","3.18999999999998","3.19999999999998","3.20999999999998","3.21999999999998","3.22999999999998","3.23999999999997","3.24999999999997","3.25999999999997","3.26999999999997","3.27999999999997","3.28999999999997","3.29999999999997","3.30999999999997","3.31999999999997","3.32999999999997","3.33999999999997","3.34999999999997","3.35999999999997","3.36999999999997","3.37999999999997","3.38999999999997","3.39999999999997","3.40999999999997","3.41999999999997","3.42999999999997","3.43999999999997","3.44999999999997","3.45999999999997","3.46999999999997","3.47999999999997","3.48999999999997","3.49999999999997","3.50999999999997","3.51999999999997","3.52999999999997","3.53999999999997","3.54999999999997","3.55999999999997","3.56999999999997","3.57999999999997","3.58999999999997","3.59999999999997","3.60999999999997","3.61999999999997","3.62999999999997","3.63999999999997","3.64999999999997","3.65999999999997","3.66999999999997","3.67999999999997","3.68999999999997","3.69999999999997","3.70999999999996","3.71999999999996","3.72999999999996","3.73999999999996","3.74999999999996","3.75999999999996","3.76999999999996","3.77999999999996","3.78999999999996","3.79999999999996","3.80999999999996","3.81999999999996","3.82999999999996","3.83999999999996","3.84999999999996","3.85999999999996","3.86999999999996","3.87999999999996","3.88999999999996","3.89999999999996","3.90999999999996","3.91999999999996","3.92999999999996","3.93999999999996","3.94999999999996","3.95999999999996","3.96999999999996","3.97999999999996","3.98999999999996","3.99999999999996","4.00999999999996","4.01999999999996","4.02999999999996","4.03999999999996","4.04999999999996","4.05999999999996","4.06999999999996","4.07999999999996","4.08999999999996","4.09999999999996","4.10999999999996","4.11999999999996","4.12999999999996","4.13999999999996","4.14999999999996","4.15999999999996","4.16999999999996","4.17999999999996","4.18999999999996","4.19999999999995","4.20999999999995","4.21999999999995","4.22999999999995","4.23999999999995","4.24999999999995","4.25999999999995","4.26999999999995","4.27999999999995","4.28999999999995","4.29999999999995","4.30999999999995","4.31999999999995","4.32999999999995","4.33999999999995","4.34999999999995","4.35999999999995","4.36999999999995","4.37999999999995","4.38999999999995","4.39999999999995","4.40999999999995","4.41999999999995","4.42999999999995","4.43999999999995","4.44999999999995","4.45999999999995","4.46999999999995","4.47999999999995","4.48999999999995","4.49999999999995","4.50999999999995","4.51999999999995","4.52999999999995","4.53999999999995","4.54999999999995","4.55999999999995","4.56999999999995","4.57999999999995","4.58999999999995","4.59999999999995","4.60999999999995","4.61999999999995","4.62999999999995","4.63999999999995","4.64999999999995","4.65999999999995","4.66999999999994","4.67999999999994","4.68999999999994","4.69999999999994","4.70999999999994","4.71999999999994","4.72999999999994","4.73999999999994","4.74999999999994","4.75999999999994","4.76999999999994","4.77999999999994","4.78999999999994","4.79999999999994","4.80999999999994","4.81999999999994","4.82999999999994","4.83999999999994","4.84999999999994","4.85999999999994","4.86999999999994","4.87999999999994","4.88999999999994","4.89999999999994","4.90999999999994","4.91999999999994","4.92999999999994","4.93999999999994","4.94999999999994","4.95999999999994","4.96999999999994","4.97999999999994","4.98999999999994","4.99999999999994","5.00999999999994","5.01999999999994","5.02999999999994","5.03999999999994","5.04999999999994","5.05999999999994","5.06999999999994","5.07999999999994","5.08999999999994","5.09999999999994","5.10999999999994","5.11999999999994","5.12999999999994","5.13999999999993","5.14999999999993","5.15999999999993","5.16999999999993","5.17999999999993","5.18999999999993","5.19999999999993","5.20999999999993","5.21999999999993","5.22999999999993","5.23999999999993","5.24999999999993","5.25999999999993","5.26999999999993","5.27999999999993","5.28999999999993","5.29999999999993","5.30999999999993","5.31999999999993","5.32999999999993","5.33999999999993","5.34999999999993","5.35999999999993","5.36999999999993","5.37999999999993","5.38999999999993","5.39999999999993","5.40999999999993","5.41999999999993","5.42999999999993","5.43999999999993","5.44999999999993","5.45999999999993","5.46999999999993","5.47999999999993","5.48999999999993","5.49999999999993","5.50999999999993","5.51999999999993","5.52999999999993","5.53999999999993","5.54999999999993","5.55999999999993","5.56999999999993","5.57999999999993","5.58999999999993","5.59999999999993","5.60999999999992","5.61999999999992","5.62999999999992","5.63999999999992","5.64999999999992","5.65999999999992","5.66999999999992","5.67999999999992","5.68999999999992","5.69999999999992","5.70999999999992","5.71999999999992","5.72999999999992","5.73999999999992","5.74999999999992","5.75999999999992","5.76999999999992","5.77999999999992","5.78999999999992","5.79999999999992","5.80999999999992","5.81999999999992","5.82999999999992","5.83999999999992","5.84999999999992","5.85999999999992","5.86999999999992","5.87999999999992","5.88999999999992","5.89999999999992","5.90999999999992","5.91999999999992","5.92999999999992","5.93999999999992","5.94999999999992","5.95999999999992","5.96999999999992","5.97999999999992","5.98999999999992","5.99999999999992","6.00999999999992","6.01999999999992","6.02999999999992","6.03999999999992","6.04999999999992","6.05999999999992","6.06999999999992","6.07999999999991","6.08999999999991","6.09999999999991","6.10999999999991","6.11999999999991","6.12999999999991","6.13999999999991","6.14999999999991","6.15999999999991","6.16999999999991","6.17999999999991","6.18999999999991","6.19999999999991","6.20999999999991","6.21999999999991","6.22999999999991","6.23999999999991","6.24999999999991","6.25999999999991","6.26999999999991","6.27999999999991","6.28999999999991","6.29999999999991","6.30999999999991","6.31999999999991","6.32999999999991","6.33999999999991","6.34999999999991","6.35999999999991","6.36999999999991","6.37999999999991","6.38999999999991","6.39999999999991","6.40999999999991","6.41999999999991","6.42999999999991","6.43999999999991","6.44999999999991","6.45999999999991","6.46999999999991","6.47999999999991","6.48999999999991","6.49999999999991","6.50999999999991","6.51999999999991","6.52999999999991","6.53999999999991","6.54999999999990","6.55999999999990","6.56999999999990","6.57999999999990","6.58999999999990","6.59999999999990","6.60999999999990","6.61999999999990","6.62999999999990","6.63999999999990","6.64999999999990","6.65999999999990","6.66999999999990","6.67999999999990","6.68999999999990","6.69999999999990","6.70999999999990","6.71999999999990","6.72999999999990","6.73999999999990","6.74999999999990","6.75999999999990","6.76999999999990","6.77999999999990","6.78999999999990","6.79999999999990","6.80999999999990","6.81999999999990","6.82999999999990","6.83999999999990","6.84999999999990","6.85999999999990","6.86999999999990","6.87999999999990","6.88999999999990","6.89999999999990","6.90999999999990","6.91999999999990","6.92999999999990","6.93999999999990","6.94999999999990","6.95999999999990","6.96999999999990","6.97999999999990","6.98999999999990","6.99999999999990","7.00999999999989","7.01999999999989","7.02999999999989","7.03999999999989","7.04999999999989","7.05999999999989","7.06999999999989","7.07999999999989","7.08999999999989","7.09999999999989","7.10999999999989","7.11999999999989","7.12999999999989","7.13999999999989","7.14999999999989","7.15999999999989","7.16999999999989","7.17999999999989","7.18999999999989","7.19999999999989","7.20999999999989","7.21999999999989","7.22999999999989","7.23999999999989","7.24999999999989","7.25999999999989","7.26999999999989","7.27999999999989","7.28999999999989","7.29999999999989","7.30999999999989","7.31999999999989","7.32999999999989","7.33999999999989","7.34999999999989","7.35999999999989","7.36999999999989","7.37999999999989","7.38999999999989","7.39999999999989","7.40999999999989","7.41999999999989","7.42999999999989","7.43999999999989","7.44999999999989","7.45999999999989","7.46999999999989","7.47999999999988","7.48999999999988","7.49999999999988","7.50999999999988","7.51999999999988","7.52999999999988","7.53999999999988","7.54999999999988","7.55999999999988","7.56999999999988","7.57999999999988","7.58999999999988","7.59999999999988","7.60999999999988","7.61999999999988","7.62999999999988","7.63999999999988","7.64999999999988","7.65999999999988","7.66999999999988","7.67999999999988","7.68999999999988","7.69999999999988","7.70999999999988","7.71999999999988","7.72999999999988","7.73999999999988","7.74999999999988","7.75999999999988","7.76999999999988","7.77999999999988","7.78999999999988","7.79999999999988","7.80999999999988","7.81999999999988","7.82999999999988","7.83999999999988","7.84999999999988","7.85999999999988","7.86999999999988","7.87999999999988","7.88999999999988","7.89999999999988","7.90999999999988","7.91999999999988","7.92999999999988","7.93999999999988","7.94999999999987","7.95999999999987","7.96999999999987","7.97999999999987","7.98999999999987","7.99999999999987","8.00999999999987","8.01999999999987","8.02999999999987","8.03999999999987","8.04999999999987","8.05999999999987","8.06999999999987","8.07999999999987","8.08999999999987","8.09999999999987","8.10999999999987","8.11999999999987","8.12999999999987","8.13999999999987","8.14999999999987","8.15999999999987","8.16999999999987","8.17999999999987","8.18999999999987","8.19999999999987","8.20999999999987","8.21999999999987","8.22999999999987","8.23999999999987","8.24999999999987","8.25999999999987","8.26999999999987","8.27999999999987","8.28999999999987","8.29999999999987","8.30999999999987","8.31999999999987","8.32999999999987","8.33999999999987","8.34999999999987","8.35999999999987","8.36999999999987","8.37999999999987","8.38999999999987","8.39999999999987","8.40999999999987","8.41999999999986","8.42999999999986","8.43999999999986","8.44999999999986","8.45999999999986","8.46999999999986","8.47999999999986","8.48999999999986","8.49999999999986","8.50999999999986","8.51999999999986","8.52999999999986","8.53999999999986","8.54999999999986","8.55999999999986","8.56999999999986","8.57999999999986","8.58999999999986","8.59999999999986","8.60999999999986","8.61999999999986","8.62999999999986","8.63999999999986","8.64999999999986","8.65999999999986","8.66999999999986","8.67999999999986","8.68999999999986","8.69999999999986","8.70999999999986","8.71999999999986","8.72999999999986","8.73999999999986","8.74999999999986","8.75999999999986","8.76999999999986","8.77999999999986","8.78999999999986","8.79999999999986","8.80999999999986","8.81999999999986","8.82999999999986","8.83999999999986","8.84999999999986","8.85999999999986","8.86999999999986","8.87999999999986","8.88999999999985","8.89999999999985","8.90999999999985","8.91999999999985","8.92999999999985","8.93999999999985","8.94999999999985","8.95999999999985","8.96999999999985","8.97999999999985","8.98999999999985","8.99999999999985","9.00999999999985","9.01999999999985","9.02999999999985","9.03999999999985","9.04999999999985","9.05999999999985","9.06999999999985","9.07999999999985","9.08999999999985","9.09999999999985","9.10999999999985","9.11999999999985","9.12999999999985","9.13999999999985","9.14999999999985","9.15999999999985","9.16999999999985","9.17999999999985","9.18999999999985","9.19999999999985","9.20999999999985","9.21999999999985","9.22999999999985","9.23999999999985","9.24999999999985","9.25999999999985","9.26999999999985","9.27999999999985","9.28999999999985","9.29999999999985","9.30999999999985","9.31999999999985","9.32999999999985","9.33999999999985","9.34999999999985","9.35999999999984","9.36999999999984","9.37999999999984","9.38999999999984","9.39999999999984","9.40999999999984","9.41999999999984","9.42999999999984","9.43999999999984","9.44999999999984","9.45999999999984","9.46999999999984","9.47999999999984","9.48999999999984","9.49999999999984","9.50999999999984","9.51999999999984","9.52999999999984","9.53999999999984","9.54999999999984","9.55999999999984","9.56999999999984","9.57999999999984","9.58999999999984","9.59999999999984","9.60999999999984","9.61999999999984","9.62999999999984","9.63999999999984","9.64999999999984","9.65999999999984","9.66999999999984","9.67999999999984","9.68999999999984","9.69999999999984","9.70999999999984","9.71999999999984","9.72999999999984","9.73999999999984","9.74999999999984","9.75999999999984","9.76999999999984","9.77999999999984","9.78999999999984","9.79999999999984","9.80999999999984","9.81999999999984","9.82999999999983","9.83999999999983","9.84999999999983","9.85999999999983","9.86999999999983","9.87999999999983","9.88999999999983","9.89999999999983","9.90999999999983","9.91999999999983","9.92999999999983","9.93999999999983","9.94999999999983","9.95999999999983","9.96999999999983","9.97999999999983","9.98999999999983","10.0000000000000"],"var":"n","width":null}],"flicker":false,"id":"c7f7feda-596c-4f67-9883-d71471e0200b","layout":[[["tau",12,null]],[["n",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠eaeeff01-8f44-47a6-ad8b-0d7766df6035s︠
︡c23f2d0d-ce33-496a-a491-3b1640b709d9︡{"done":true}︡
︠e1c3d88d-7492-41f5-9d50-4f17f2a19acbs︠
︡1142399d-75c8-4d18-b013-21801d0a987b︡{"done":true}︡









